/*
    Magnifier II - An image viewer with a magnifier
    Copyright © 2014-2017 Harry Whitfield

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the Free
    Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    MA  02110-1301  USA

    Magnifier II - web widget version 1.0
    23 November, 2017
    Copyright © 2014-2017 Harry Whitfield
    mailto:g6auc@arrl.net
*/

/*jslint browser, devel */

/*global window, document, File, FileReader, FileList, Blob,
    newFileSelector, newDiv, newImage, newCanvas, moveObj, left, vtop, moveDiv,
    setPreference, Slider, scale, Image
*/

/*property
    PI, addEventListener, appendChild, arc, beginPath, body, clientX, clientY,
    clip, ctx, dataTransfer, div, drawImage, dropEffect, files, floor,
    getElementById, innerHTML, lastModified, left, match, name, naturalHeight,
    naturalWidth, onchange, onload, onmousedown, onmouseleave, onmousemove,
    onmouseup, ontouchend, ontouchmove, ontouchstart, opacity, open,
    preventDefault, readAsDataURL, removeChild, restore, result, save, scale,
    scrollLeft, scrollTop, scrollX, scrollY, selectedIndex, setValue, size,
    src, stopPropagation, style, targetTouches, title, toFixed, top,
    touchcancel, translate, type, value
*/

var handleFiles = function (fileInput, fileInfo, imageArea, callback) {
    "use strict";
    var handleFile = function (file) {
        fileInfo.innerHTML = file.name + ", " + file.size + " bytes, Type: " + file.type + ", Modified: " + new Date(file.lastModified);

        if (file.type.match(/^image\//)) {
            var reader = new FileReader();

            reader.onload = function () {
                var img = new Image();
                var delay = function () {
                    callback(img);
                };

                imageArea.innerHTML = "";
                img.src = reader.result;
                imageArea.appendChild(img);
                setTimeout(delay, 1000);
            };
            reader.readAsDataURL(file);
        } else {
            imageArea.innerHTML = "File is not an image file!";
        }
    };

    var handleDrop = function (evt) {
        var files = evt.dataTransfer.files;
        var file = files[0];

        evt.stopPropagation();
        evt.preventDefault();
        handleFile(file);
    };

    var handleDragOver = function (evt) {
        evt.stopPropagation();
        evt.preventDefault();
        evt.dataTransfer.dropEffect = "copy";
    };

    var handleSelect = function (evt) {
        var file = fileInput.files[0];

        evt.stopPropagation();
        evt.preventDefault();
        handleFile(file);
    };

    if (File && FileReader && FileList && Blob) {
        imageArea.addEventListener("dragover", handleDragOver, false);
        imageArea.addEventListener("drop", handleDrop, false);
        fileInput.addEventListener("change", handleSelect, false);

    } else {
        alert("File APIs are missing.");
    }
};

var $ = function (id) {
    "use strict";
    return document.getElementById(id);
};

var left = 250; // global - sets the width of the left margin
var lensScalePref;
var magScalePref;
var lensTypePref;
var ghStart = 150;
var gvStart = 80;
var ghOffset = ghStart;
var gvOffset = gvStart;

window.onload = function () {
    "use strict";
    var magWidth = 540;
    var magHeight = 502;
    var lensScale = Number(lensScalePref) / 100;
    var magScale = Number(magScalePref);
    var radius = Math.floor(144 * lensScale);   // radius of the lens
    var magX = Math.floor(379 * lensScale);     // centre X point of lens
    var magY = Math.floor(148 * lensScale);     // centre Y point of lens
    var handleX = Math.floor(140 * lensScale);  // centre X point of handle
    var handleY = Math.floor(350 * lensScale);  // centre Y point of handle
    var isMouseDown = false;
    var tracking = radius;

    var fileSelectorY = 0;
    var fileInfoY = 30;
    var coordInfoY = 60;
    var testInfoY = 90;
    var helpInfoY = 90;
    var slider1X = 300;
    var slider2X = 600;
    var slidersY = 120;
    var imageAreaY = 160;

//  var defaultImage = "<img src=\"Resources/Images/elektra.jpg\">";
    var gImage = null;
    var gWidth = 640;   // initially for defaultImage
    var gHeight = 480;

    var fileInput = newFileSelector(0, fileSelectorY, 200, 6.3, "", 1);
    var helpButton = newImage(0, helpInfoY, 30, 22, "Resources/Images/help.png", 1);
    var fileInfo = newDiv(0, fileInfoY, 1, "fileInfo");
    var coordInfo = newDiv(0, coordInfoY, 1, "coordInfo");
    var testInfo1 = newDiv(slider1X, testInfoY, 1, "testInfo1");
    var testInfo2 = newDiv(slider2X, testInfoY, 1, "testInfo2");

    var imageArea = newDiv(0, imageAreaY, 1, "imageArea");
    var blankArea = newDiv(0, imageAreaY + gHeight, 1, "blankArea");
    var magnifier = null;
    var lens = null;
    var lensType = lensTypePref;
    var ctx;

    helpButton.title = "Displays information about this program.";

    var magnify = function (xPoint, yPoint) {
        var scrollX = window.scrollX;
        var scrollY = window.scrollY;

        ctx.save();
        ctx.translate(magX, magY);      // centre point of lens
        ctx.scale(radius, radius);      // scale to radius of lens
        ctx.beginPath();
        ctx.arc(0, 0, 1.0, 0, 2 * Math.PI, false);  // unit circle
        ctx.clip();                     // clipping region
        ctx.scale(1 / radius, 1 / radius);
        ctx.drawImage(
            gImage,
            (-xPoint - scrollX) * magScale,
            (-yPoint - scrollY) * magScale,
            magScale * gWidth,          // gImage.naturalWidth,
            magScale * gHeight          // gImage.naturalHeight
        );
        ctx.restore();
    };

    var doMove = function (hOffset, vOffset) {
        var scrollX = window.scrollX;
        var scrollY = window.scrollY;

    	ghOffset = hOffset + magX;
		gvOffset = vOffset + magY - imageAreaY;

        moveObj(magnifier, hOffset, vOffset);

        coordInfo.innerHTML = "[" + (hOffset + magX) + ", " + (vOffset + magY - imageAreaY) + "]";
        coordInfo.innerHTML += (
        	" (" + scrollX + ", " + scrollY + ") (" +
            imageArea.scrollLeft + ", " + imageArea.scrollTop + ")" +
            " width: " + gWidth + " x height: " + gHeight
        );

        if (gImage !== null) {
            magnify(hOffset + magX - scrollX, vOffset + magY - imageAreaY - scrollY);
        }
        moveObj(lens, hOffset, vOffset);
    };

    var callback = function (img) {
        gImage = img;
        gWidth = img.naturalWidth;
        gHeight = img.naturalHeight;
        if ((gWidth < 100) || (gHeight < 100)) {
            alert("Loading failed - try again! " + gWidth + ", " + gHeight);
        }
        doMove(ghStart - magX, gvStart - magY + imageAreaY); // position lens at [150, 80] initially
        moveDiv(blankArea, 0, imageAreaY + gHeight);
    };

    var makeLens = function () {
        if (lens !== null) {
            document.body.removeChild(lens.div);
        }

        radius = Math.floor(144 * lensScale);   // radius of the lens
        magX = Math.floor(379 * lensScale);     // centre X point of lens
        magY = Math.floor(148 * lensScale);     // centre Y point of lens
        handleX = Math.floor(140 * lensScale);  // centre X point of handle
        handleY = Math.floor(350 * lensScale);  // centre Y point of handle
        tracking = radius;

        lens = newCanvas(0, imageAreaY, magWidth * lensScale, magHeight * lensScale, "", 5, 1.0);
        ctx = lens.ctx;
    };

    var makeMagnifier = function () {
        var magLens = (
        	lensType === "1"
        	? "magnifier.png"
        	: "magnifier_plain.png"
        );

        if (magnifier !== null) {
            document.body.removeChild(magnifier.div);
        }
        magnifier = newImage(0, imageAreaY, magWidth * lensScale, magHeight * lensScale, "Resources/Magnifier/" + magLens, 10, 1.0);

        magnifier.onmousedown = function (evt) {
            evt.stopPropagation();
            evt.preventDefault();
            isMouseDown = true;
        };

        magnifier.onmouseup = function (evt) {
            evt.stopPropagation();
            evt.preventDefault();
            isMouseDown = false;
        };

        magnifier.onmouseleave = function (evt) {
            evt.stopPropagation();
            evt.preventDefault();
            isMouseDown = false;
        };

    	magnifier.ontouchstart = function (evt) {
            evt.stopPropagation();
            evt.preventDefault();
            isMouseDown = true;
        };

		magnifier.ontouchend = function (evt) {
            evt.stopPropagation();
            evt.preventDefault();
            isMouseDown = false;
        };

		magnifier.touchcancel = function (evt) {
            evt.stopPropagation();
            evt.preventDefault();
            isMouseDown = false;
        };

        var magnifierMove = function (posX, posY) {
            var hOffset;
            var vOffset;
            var scrollX = window.scrollX;
            var scrollY = window.scrollY;

            if (isMouseDown) {
                hOffset = magnifier.left + posX - handleX - left + scrollX;
                vOffset = magnifier.top + posY - handleY - vtop - imageAreaY + scrollY;

                if (hOffset < -magX + tracking) {
                    hOffset = -magX + tracking;
                }
                if (hOffset > gWidth - magX - tracking) {
                    hOffset = gWidth - magX - tracking;
                }

                if (vOffset < -magY + imageAreaY + tracking) {
                    vOffset = -magY + imageAreaY + tracking;
                }
                if (vOffset > gHeight - magY + imageAreaY - tracking) {
                    vOffset = gHeight - magY + imageAreaY - tracking;
                }

                doMove(hOffset, vOffset);
            }
        };

        magnifier.onmousemove = function (evt) {
        	var posX;
        	var posY;

            evt.stopPropagation();
            evt.preventDefault();

      		posX = evt.clientX;
        	posY = evt.clientY;
        	magnifierMove(posX, posY);
        };

        magnifier.ontouchmove = function (evt) {
        	var posX;
        	var posY;

            evt.stopPropagation();
            evt.preventDefault();

      		posX = evt.targetTouches[0].clientX;
        	posY = evt.targetTouches[0].clientY;
        	magnifierMove(posX, posY);
        };
    };

    var callback1 = function (value) {
       	var tmp;

       	lensScale = (0.85 * value + 30) / 200;
       	tmp = 100 * lensScale;
       	testInfo1.innerHTML = "Magnifier Size: " + tmp.toFixed(0) + "%";
        makeLens();
        makeMagnifier();
//      doMove(150 - magX, 80 - magY + imageAreaY); // position lens at [150, 80]
        doMove(ghOffset - magX, gvOffset - magY + imageAreaY); // position lens at [150, 80]
    };

    var callback2 = function (value) {
		var tmp;

		if (value >= 100) {
			tmp = 1.0 + 3 * (value - 100) / 100;
		} else {
			tmp = 0.25 + 0.75 * value / 100;
		}
		magScale = tmp;
		tmp = 100 * tmp;
		testInfo2.innerHTML = "Magnification: " + tmp.toFixed(0) + "%";
//      doMove(150 - magX, 80 - magY + imageAreaY); // position lens at [150, 80]
        doMove(ghOffset - magX, gvOffset - magY + imageAreaY); // position lens at [150, 80]
    };

	helpButton.onmousedown = function () {
    	helpButton.style.opacity = "0.5";
	};
	helpButton.onmouseup = function () {
    	helpButton.style.opacity = "1.0";
    	window.open("Resources/Help.html");
	};

    var slider1 = new Slider("horizontal", slider1X, slidersY, 200, 20, left, vtop, "Resources/", 10, scale, callback1);
    var slider2 = new Slider("horizontal", slider2X, slidersY, 200, 20, left, vtop, "Resources/", 10, scale, callback2);

    var MenuTitleArray = [
        "Magnifier Size",
        "Magnification",
        "Magnifier Lens Type"
    ];

    var MenuSelectedIndex = [5, 1, 0]; // 50%, 2x and Normal
    var Menu = [];

    Menu[0] = $("selector0");   // defined in html file
    Menu[0].selectedIndex = MenuSelectedIndex[0];
    Menu[0].title = MenuTitleArray[0];

    Menu[1] = $("selector1");   // defined in html file
    Menu[1].selectedIndex = MenuSelectedIndex[1];
    Menu[1].title = MenuTitleArray[1];

    Menu[2] = $("selector2");   // defined in html file
    Menu[2].selectedIndex = MenuSelectedIndex[2];
    Menu[2].title = MenuTitleArray[2];

    Menu[0].onchange = function () {
        var scale0 = Menu[0].value;
        var value;
        var tmp;

        setPreference("lensScalePref", scale0);
       	lensScale = Number(scale0) / 100;

       	value = (200 * lensScale - 30) / 0.85;
       	slider1.setValue(value);
       	tmp = 100 * lensScale;
       	testInfo1.innerHTML = "Magnifier Size: " + tmp.toFixed(0) + "%";

        makeLens();
        makeMagnifier();
//      doMove(150 - magX, 80 - magY + imageAreaY); // position lens at [150, 80]
        doMove(ghOffset - magX, gvOffset - magY + imageAreaY); // position lens at [150, 80]
    };

    Menu[1].onchange = function () {
    	var scale1 = Menu[1].value;
    	var value;
    	var tmp;

		setPreference("magScalePref", scale1);
        magScale = Number(scale1);

        if (magScale < 1.0) {
        	value = 100 * (magScale - 0.25) / 0.75;
        } else {
        	value = 100 * (magScale - 1.0) / 3 + 100;
        }
        slider2.setValue(value);
		tmp = 100 * magScale;
        testInfo2.innerHTML = "Magnification: " + tmp.toFixed(0) + "%";
//      doMove(150 - magX, 80 - magY + imageAreaY); // position lens at [150, 80]
        doMove(ghOffset - magX, gvOffset - magY + imageAreaY); // position lens at [150, 80]
    };

    Menu[2].onchange = function () {
    	var type2 = Menu[2].value;

 		setPreference("lensTypePref", type2);
        lensType = type2;
        makeLens();
        makeMagnifier();
//      doMove(150 - magX, 80 - magY + imageAreaY); // position lens at [150, 80]
        doMove(ghOffset - magX, gvOffset - magY + imageAreaY); // position lens at [150, 80]
    };

	var setMenus = function () {
    	var value;
    	var tmp;
    	var lensScaleX;

    	Menu[0].value = lensScalePref;
    	Menu[1].value = magScalePref;
    	Menu[2].value = lensTypePref;

		lensScaleX = Number(lensScalePref) / 100;

    	value = (200 * lensScaleX - 30) / 0.85;
       	slider1.setValue(value);
       	tmp = Number(lensScalePref);
       	testInfo1.innerHTML = "Magnifier Size: " + tmp.toFixed(0) + "%";

        if (magScale < 1.0) {
        	value = 100 * (magScale - 0.25) / 0.75;
        } else {
        	value = 100 * (magScale - 1.0) / 3 + 100;
        }
        slider2.setValue(value);
		tmp = 100 * Number(magScalePref);
        testInfo2.innerHTML = "Magnification: " + tmp.toFixed(0) + "%";
	};

    handleFiles(fileInput, fileInfo, imageArea, callback);
    fileInfo.innerHTML = "elektra.jpg, 117427 bytes, Type: image/jpeg, Modified: Thu Jun 5 2017 22:30:00 GMT+0000 (GMT)";

    var start = function () {
        doMove(ghStart - magX, gvStart - magY + imageAreaY); // position lens at [150, 80]
    };

    gImage = new Image();
    gImage.src = "Resources/Images/elektra.jpg";
    imageArea.appendChild(gImage);

    setMenus();
    makeLens();
    makeMagnifier();
	setTimeout(start, 500);
};
