/*
    Magnifier II - An image viewer with a magnifier
    Copyright © 2014-2017 Harry Whitfield

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the Free
    Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    MA  02110-1301  USA

    Magnifier II - web widget version 1.0 with world file functions
    22 November, 2017
    Copyright © 2014-2017 Harry Whitfield
    mailto:g6auc@arrl.net
*/

/*jslint browser */

/*global require */

/*property
    appendFileSync, encoding, homedir, join, platform, readFile, readFileSync,
    split, writeFile, writeFileSync
*/

var wpFolder = (function () {
	"use strict";
	var os;

	// check for availability of filesystem functions in node.js
	if (typeof require === "function" && require("fs") !== undefined) {
		os = require("os");
		return (
			os.platform() === "darwin"
			? os.homedir() + "/Desktop"
			: null
		);							// needs a windows path
	}
	return null;
}());

// define the filesystem function
var filesystem = (function () {
	"use strict";
	var itself;
	var fs;
	var os;
	var system;

	// check for availability of filesystem functions in node.js
	if (typeof require === "function" && require("fs") !== undefined) {
		itself = {};
		fs = require("fs");
		os = require("os");
		system = {};

		system.platform = (
			os.platform() === "darwin"
			? "macintosh"
			: "windows"
		);

		itself.readFile = function (path, asLines) {
	    	var text = fs.readFileSync(path, {encoding: "utf8"});

	 		return (
	 			asLines
	 			? text.split(/\r\n?|\n/)
	 			: text
	 		);
		};

		itself.writeFile = function (path, text, append) {
			if (typeof text === "string") {
				text = text.split(/\r\n?|\n/);
			}
			text = (
				system.platform === "macintosh"
				? text.join("\n")
				: text.join("\r\n")
			);

			if (append) {
				fs.appendFileSync(path, text, {encoding: "utf8"});
			} else {
				fs.writeFileSync(path, text, {encoding: "utf8"});
			}
		};
		return itself;
	}
	return null;
}());
