/*
	Web Storage for Preferences
	Copyright © 2019 Harry Whitfield

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
	MA	02110-1301	USA

	Web Storage for Preferences - browser version 1.0
	26 April, 2019
	Copyright © 2019 Harry Whitfield
	mailto:g6auc@arrl.net
*/

/*jslint browser */

/*property
	getItem, removeItem, setItem
*/

var widgetName = "Magnifier_II Web Widget";

function setPreference(key, value) {
	if (Storage !== undefined) {
		localStorage.setItem(widgetName + ":" + key, value);
	}
}

function getPreference(key) {
	if (Storage !== undefined) {
		return localStorage.getItem(widgetName + ":" + key);
	}
	return null;
}

function removePreference(key) {
	if (Storage !== undefined) {
		localStorage.removeItem(widgetName + ":" + key);
	}
}

///////////////////////////////////// Preference Code ////////////////////////////////////

var lensScalePref = "50";	// was 0.5
var magScalePref = "2.0";	// was 2.0
var lensTypePref = "1";

var utmZonePref = "31";			// All strings
var hemispherePref = "Northern";
var coordinatesPref = "0";
var datumPref = "WGS84";

function getPreferences() {
	"use strict";
	var lensScale;
	var magScale;
	var lensType;

	var utmZone;
	var hemisphere;
	var coordinates;
	var datum;

	lensScale = getPreference("lensScalePref");
	if (lensScale !== null) {
		lensScalePref = lensScale;	// was Number(lensScale) / 100;
	}

	magScale = getPreference("magScalePref");
	if (magScale !== null) {
		magScalePref = magScale;	// was Number(magScale);
	}

	lensType = getPreference("lensTypePref");
	if (lensType !== null) {
		lensTypePref = lensType;
	}

	utmZone = getPreference("utmZonePref");
	if (utmZone !== null) {
		utmZonePref = utmZone;
	}

	hemisphere = getPreference("hemispherePref");
	if (hemisphere !== null) {
		hemispherePref = hemisphere;
	}

	coordinates = getPreference("coordinatesPref");
	if (coordinates !== null) {
		coordinatesPref = coordinates;
	}

	datum = getPreference("datumPref");
	if (datum !== null) {
		datumPref = datum;
	}
}

getPreferences();				// pick up preferences stored in web storage
